# Copyright 2022-present Lenovo
# Confidential and Proprietary
import os
import sys
from xml.etree.ElementTree import Element, ElementTree


def parse_xml(xml_path):
    xml = ElementTree(file=xml_path)
    root = xml.getroot()
    filename = root.find("filename").text
    labeled = False
    objects = root.findall("object")
    if objects:
        labeled = True
    return filename, labeled


def reformat_and_save_xml(old_xml_path, new_xml_path, img_path):
    try:
        xml = ElementTree(file=old_xml_path)
        root = xml.getroot()
        labels = []
        objects = root.findall("object")
        for obj in objects:
            obj.remove(obj.find("truncated"))
            obj.remove(obj.find("occluded"))
            labels.append(obj.find("name").text)
            xmin = obj.find("bndbox").find("xmin")
            xmin.text = str(round(float(xmin.text)))
            ymin = obj.find("bndbox").find("ymin")
            ymin.text = str(round(float(ymin.text)))
            xmax = obj.find("bndbox").find("xmax")
            xmax.text = str(round(float(xmax.text)))
            ymax = obj.find("bndbox").find("ymax")
            ymax.text = str(round(float(ymax.text)))
        root.remove(root.find("source"))
        root.remove(root.find("segmented"))
        pose = Element('pose')
        pose.text = 'Unspecified'
        root.append(pose)
        truncated = Element('truncated')
        truncated.text = '0'
        root.append(truncated)
        occluded = Element('occluded')
        occluded.text = '0'
        root.append(occluded)
        root.find('folder').text = 'images'
        root.find('filename').text = img_path
        root.find('size').find('depth').text = '3'
        xml.write(new_xml_path, encoding="utf-8", xml_declaration=True)
    except Exception as e:
        remove_file([new_xml_path])
        raise Exception(f"Convert {old_xml_path} error : {e}")
    else:
        return labels


def get_dest_filename(dest_file_path, thumbnail_path):
    thumbnail_path_ext = '.gif'
    dest_path_dir, dest_name = os.path.split(dest_file_path)
    if " " in dest_name:
        print("[ERROR] Illegal characters exist in the file "
              "name %s." % dest_name)
        return False, ""
    picture, ext = os.path.splitext(dest_name)
    thumbnail_file_path = os.path.join(thumbnail_path, "%s%s" % (
        picture, thumbnail_path_ext))
    num = 0
    while 1:
        if os.path.exists(thumbnail_file_path):
            num += 1
            new_picture = "%s(%s)%s" % (picture, num, thumbnail_path_ext)
            thumbnail_file_path = os.path.join(thumbnail_path, new_picture)
        else:
            if num == 0:
                break
            thumbnail_name = os.path.basename(thumbnail_file_path)
            new_dest_name = "%s%s" % (
                os.path.splitext(thumbnail_name)[0], ext)
            dest_file_path = os.path.join(
                dest_path_dir, new_dest_name)
            print("[RENAME] File %s already exists, rename it "
                  "to %s" % (dest_name, new_dest_name))
            break
    return True, dest_file_path


def remove_file(file_list):
    for file in file_list:
        try:
            if os.path.exists(file):
                os.remove(file)
        except Exception as e:
            print("[Error]: Failed to remove file %s. "
                  "Reason:%s" % (os.path.basename(file), e))


def delete_last_line(file_path):
    try:
        with open(file_path, 'rb+') as f:
            lines = f.readlines()
            if len(lines) >= 2:
                seek_num = len(lines[-1]) + 2
            else:
                seek_num = len(lines[-1])
            f.seek(-seek_num, os.SEEK_END)
            f.truncate()
    except Exception as e:
        print("Failed to delete the last line "
              "of file %s! Reason: %s" % (os.path.basename(file_path), e))
        sys.exit(1)
