# Copyright 2021-present Lenovo
# Confidential and Proprietary

import copy
import os
import sys

import tensorflow.compat.v1 as tf
tf.disable_v2_behavior()
from PIL import Image
from tensorflow.python.platform import gfile

parentpath = '/opt/letrain/'
path_list = [parentpath,
             parentpath + 'lib',
             parentpath + 'engine',
             parentpath + 'engine/automl/evaluator',
             parentpath + 'tfserving',
             parentpath + 'inference',
             parentpath + 'applications',
             os.path.join(parentpath, "applications/frcnn")]
sys.path += path_list

from engine.automl.learner.modelset.tfefficientdet import hparams_config
from engine.automl.learner.modelset.tfefficientdet.inference import (
    build_inputs_images, build_model, det_post_process)

sys.path += ['/opt/letrain/applications/frcnn/lib/utils/']

from autoML_common import restore_ckpt, visualize_image
from timer import Timer

tf.app.flags.DEFINE_string('debug', 'No', '')

FLAGS = tf.app.flags.FLAGS


class AutoDetectionDeploySer(object):
    def load_model(self, model_path, model_name):
        self.LABELS_FILE = os.path.join(model_path, 'category.txt')

        self.model_name = None
        try:
            with open(os.path.join(model_path, "fix_gear.txt"), 'r') as f:
                fix_gear = f.readlines()[0]
                self.model_name = "efficientdet-d" + str(int(fix_gear) - 1)
        except Exception as e:
            print("Failed to get fix gear. \n{0}".format(e), flush=True)

        self.pre_check(model_path)

        self.MODEL_DIR = model_path
        self.sess = tf.Session(
            config=tf.ConfigProto(allow_soft_placement=True))
        init = tf.global_variables_initializer()
        self.sess.run(init)

        self.label_id_mapping = {}
        with open(self.LABELS_FILE, 'r') as data:
            for ID, name in enumerate(data):
                self.label_id_mapping[ID] = name.strip('\n')
        self.num_classes = len(self.label_id_mapping)

        self.batch_size = 1
        self.params = hparams_config.get_detection_config(
            self.model_name).as_dict()
        if self.num_classes:
            self.params.update(dict(num_classes=self.num_classes))
        params = copy.deepcopy(self.params)
        self.kwargs = {'max_boxes_to_draw': 100, 'min_score_thresh': 0.4}
        self.disable_pyfun = False

        self.input_image = tf.placeholder(
            tf.float32,
            [1, self.params['image_size'], self.params['image_size'], 3])
        self.input_scales = tf.placeholder(dtype=tf.float32)
        class_outputs, box_outputs = build_model(
            self.model_name, self.input_image, **self.params)
        restore_ckpt(self.sess, self.MODEL_DIR, True, None)
        # for postprocessing.
        params.update(
            dict(batch_size=self.batch_size, disable_pyfun=self.disable_pyfun))
        # Build postprocessing.
        self.detections_batch = det_post_process(
            params,
            class_outputs,
            box_outputs,
            self.input_scales,
            min_score_thresh=self.kwargs.get('min_score_thresh', 0.2),
            max_boxes_to_draw=self.kwargs.get('max_boxes_to_draw', 50))

    def pre_check(self, model_path):
        if not self.model_name:
            raise ValueError("Inference model name must be provided.")
        if not model_path:
            raise ValueError(
                "Checkpoint dir must be provided to restore model."
            )
        if not gfile.Exists(self.LABELS_FILE):
            raise ValueError("label file: %s is not found." % self.LABELS_FILE)

    def inference(self, image_file_name):
        params = copy.deepcopy(self.params)
        image_path_pattern = [image_file_name]
        labels = [0] * len(image_path_pattern)

        timer = Timer()
        timer.tic()

        with tf.Graph().as_default(), tf.Session() as sess:
            # Buid inputs and preprocessing.
            raw_images, images, scales = build_inputs_images(
                image_path_pattern, labels,
                params['image_size'], batch_size=self.batch_size)
            new_images, new_scales = sess.run([images, scales])

        outputs_np = self.sess.run(
            self.detections_batch,
            feed_dict={self.input_image: new_images,
                       self.input_scales: new_scales})
        timer.toc()
        detection_res = []
        # Visualize results.
        for i, output_np in enumerate(outputs_np):
            # output_np has format
            # [image_id, y, x, height, width, score, class]
            boxes = output_np[:, 1:5]
            classes = output_np[:, 6].astype(int)
            scores = output_np[:, 5]

            # This is not needed if disable_pyfun=True
            # convert [x, y, width, height] to [ymin, xmin, ymax, xmax]
            # TODO(tanmingxing): make this convertion more efficient.
            if not self.disable_pyfun:
                boxes[:, [0, 1, 2, 3]] = boxes[:, [1, 0, 3, 2]]

            boxes[:, 2:4] += boxes[:, 0:2]
            img, detection_res = visualize_image(
                raw_images[0][i],
                boxes,
                classes,
                scores,
                self.label_id_mapping,
                **self.kwargs)

            # If the image name ends with '.jpg', '.png' or '.jepg ',
            # which will be visualized successfully. Otherwise, it may fail.
            try:
                Image.fromarray(img).save(image_file_name)
            except Exception:
                print("Failed to visualize image.")

        automl_object_res = {
            "duration": int(round(1000 * timer.total_time)),
            "data": detection_res}

        return automl_object_res


if __name__ == '__main__':
    model_path = "/for_autoML/trt/new_deploy_service/ob"
    model_name = "efficientdet-d3"
    image_file_name = "/for_autoML_deploy_service/ob/test_app/image/fruit0.jpg"
    detection_deploy_ser = AutoDetectionDeploySer()
    detection_deploy_ser.load_model(model_path, model_name)
    print(detection_deploy_ser.inference(image_file_name))
