# Copyright 2018-present Lenovo
# Confidential and Proprietary

import os
import sys

import numpy as np
import tensorflow.compat.v1 as tf
tf.disable_v2_behavior()
from tensorflow.python.eager import context
from tensorflow.python.platform import gfile

parentpath = '/opt/letrain/'
path_list = [parentpath,
             parentpath + 'lib',
             parentpath + 'engine',
             parentpath + 'inference',
             parentpath + 'applications',
             os.path.join(parentpath, "applications/frcnn")]
sys.path += path_list

# import input_reader_orig as input_reader  # noqa: F401
import letrain  # noqa: F401
from lib.utils.timer import Timer
from nets import nets_factory
from preprocessing import image_preprocessing, preprocessing_factory

tf.logging.set_verbosity(tf.logging.WARN)
FLAGS = tf.app.flags.FLAGS


def get_image(_data, height, width, _fn):
    if FLAGS.data_format == 'raw_image' and FLAGS.task == 'custom':
        resize_side = int(height * 1.15)
        image = image_preprocessing.preprocess_image(
            _data.uint8image, height, width, resize_side=resize_side
        )
    elif FLAGS.data_format == 'tfrecord' and FLAGS.task == 'custom':
        raise Exception("ERROR custom data not support tfrecrd data")
    else:
        image = _fn(_data.uint8image, height, width)
    return image


class ClassificationDeploySer(object):
    def load_model(self, model_path, model_name):
        # -------------------pre_check
        "pre check all flags to make sure following work can be executed."
        self.LABELS_FILE = model_path + '/labels.txt'
        self.model_name = model_name
        self.pre_check(model_path)
        self.g = tf.Graph()
        with self.g.as_default():
            self.infer_device = self.get_available_device()
            with tf.device(self.infer_device):
                # Get target label doc for matching probability
                self.labels_dict = {}
                try:
                    with open(self.LABELS_FILE) as f:
                        for line in f:
                            key, value = line.rstrip('\n').split(':')
                            self.labels_dict[key] = value
                except IOError:
                    raise ValueError("Can't open file:", self.LABELS_FILE)

                # Get model
                num_classes = len(self.labels_dict)
                network_fn = nets_factory.get_network_fn(self.model_name,
                                                         num_classes,
                                                         is_training=False)
                self.image_size = network_fn.default_image_size

                self.image_place = tf.placeholder(
                    tf.float32, [1, self.image_size, self.image_size, 3]
                )
                self.logits, _ = network_fn(self.image_place)

                # Calculate probabilities with softmax activation
                self.prob = tf.nn.softmax(self.logits)

                # Create session
                gpu_options = tf.GPUOptions(per_process_gpu_memory_fraction=0.3)
                self.sess = tf.Session(
                    config=tf.ConfigProto(
                        gpu_options=gpu_options,
                        allow_soft_placement=True,
                        log_device_placement=False
                    )
                )

                # Restore variables from checkpoint file into graph
                saver = tf.train.Saver()
                ckpt = tf.train.latest_checkpoint(model_path)
                saver.restore(self.sess, ckpt)

    def inference(self, image_file_name):
        with self.g.as_default():
            self.infer_device = self.get_available_device()
            with tf.device(self.infer_device):
                # Code below are used for actual op compute
                timer = Timer()
                timer.tic()
            image = self.preprocess_file(
                image_file_name, self.image_size, self.image_size
            )
            logits_val, prob_val = self.sess.run(
                [self.logits, self.prob],
                feed_dict={self.image_place: self.sess.run(image)}
            )
            timer.toc()
            classification_result = {}
            if isinstance(prob_val, np.ndarray):
                classification_data = []

                # Take only the best results
                # index = np.argmax(prob_val, axis=1)

                for i, j in enumerate(prob_val[0]):
                    result = {
                        'label': self.labels_dict[str(i)],
                        'probability': round(float(j), 4)
                    }
                    classification_data.append(result)
                classification_data = sorted(classification_data,
                                             key=lambda x: x['probability'],
                                             reverse=True)[0:5]

                # parse the classification infernece results
                classification_result['data'] = classification_data
                classification_result['duration'] = int(
                    round(1000 * timer.total_time))
            else:
                raise ValueError("Unknown format probability result. ")

        return classification_result

    def get_available_device(self):
        num_gpu = context.num_gpus()
        return '/device:GPU:0' if num_gpu else '/device:CPU:0'

    def pre_check(self, model_path):
        if not self.model_name:
            raise ValueError("Inference model name must be provided.")
        if not model_path:
            raise ValueError(
                "Checkpoint dir must be provided to restore model."
            )
        if not gfile.Exists(self.LABELS_FILE):
            raise ValueError("label file: %s is not found." % self.LABELS_FILE)

    def preprocess_file(self, filename, height, width):
        """Preprocess data from single image file

        Args:
            filename: image file name (full path)
            height: expected file size
            width: expected file size

        Return:
            image with expected height/width using corresponding nets model.
        """

        class Record(object):
            pass

        # decode img
        _data = Record()
        _data.name = filename
        file_contents = tf.read_file(_data.name)
        record_bytes = tf.image.decode_image(
            file_contents, channels=3, name='image_decode')
        _data.uint8image = record_bytes

        _name = self.model_name
        _fn = preprocessing_factory.get_preprocessing(_name, is_training=False)
        image = get_image(_data, height, width, _fn)
        return tf.reshape(image, [1, height, width, 3])

    def preprocess_filelist(self, filenamelist, height, width, num_batch):
        """Preprocess data from image file list

        Args:
            filename_list: image file name list (full path)
            height: expected file size
            width: expected file size
            num_batch: number of files in filename_list

        Return:
            images with expected height/width using corresponding nets model.
        """

        class Record(object):
            pass

        _data_list = []
        for i in range(num_batch):
            filename = filenamelist[i]
            result = Record()
            result.name = filename
            file_contents = tf.read_file(result.name)
            record_bytes = tf.image.decode_image(
                file_contents, channels=3, name='image_decode')
            result.uint8image = record_bytes
            _data_list.append(result)

        _name = self.model_name
        _reshaped_img_list = []
        _fn = preprocessing_factory.get_preprocessing(_name, is_training=False)
        for _data in _data_list:
            image = get_image(_data, height, width, _fn)
            image = tf.reshape(image, [height, width, 3])
            _reshaped_img_list.append(image)
        return tf.stack(_reshaped_img_list)


if __name__ == "__main__":
    model_path = '/home/hpcadmin/ai_studio/tasks/' \
                 'edc9f53c-1acf-11e9-9d76-7cd30ae2c038/test_demo_0/train_dir'
    model_name = 'resnet_v2_50'

    image_file_name = '/home/hpcadmin/ex_test/' \
                      'test_validation/class_inference_test/tensorflow.jpg'
    response_dir = '/home/hpcadmin/ex_test/' \
                   'test_validation/class_inference_test'

    class_obj = ClassificationDeploySer()
    class_obj.load_model(model_path, model_name)
    class_obj.inference(image_file_name)
