# Copyright 2022-present Lenovo
# Confidential and Proprietary
import os
import sys
from shutil import copyfile

import tensorflow.compat.v1 as tf
tf.disable_v2_behavior()
from tensorflow.python.platform import gfile
from export_model import BaseExportService

parentpath = '/opt/letrain/'
path_list = [parentpath,
             parentpath + 'lib',
             parentpath + 'engine',
             parentpath + 'inference',
             parentpath + 'applications',
             os.path.join(parentpath, "applications/yolov3")]
sys.path += path_list

tf.app.flags.DEFINE_boolean('inference', True, 'hard code in yolo core utils')
tf.app.flags.DEFINE_string('dataset_types', 'jepg', 'hard code in yolo core '
                                                    'utils')
tf.app.flags.DEFINE_string("checkpoint_dir", "checkpoint", "")
FLAGS = tf.app.flags.FLAGS
FLAGS.checkpoint_dir = FLAGS.lico_checkpoint_dir

from core.yolov3 import YOLOV3

LABELS_FILE = os.path.join(FLAGS.lico_checkpoint_dir, 'category.txt')
VERSIONCONTROLL = FLAGS.lico_version_control


class YOLOv3ExportService(BaseExportService):
    def __init__(self):
        self.pre_check()
        if not gfile.Exists(LABELS_FILE):
            raise ValueError("label file: %s is not found." % LABELS_FILE)

    def save_pb(self, sess, export_path_base, graph):
        export_path = self.get_export_path(
            export_path_base,
            VERSIONCONTROLL,
            FLAGS.model_version
        )

        builder = tf.saved_model.builder.SavedModelBuilder(export_path)

        image_inputs = tf.saved_model.utils.build_tensor_info(
            graph.get_tensor_by_name("input/input_data:0")
        )
        sbbox = tf.saved_model.utils.build_tensor_info(
            graph.get_tensor_by_name("pred_sbbox/concat_2:0")
        )
        mbbox = tf.saved_model.utils.build_tensor_info(
            graph.get_tensor_by_name('pred_mbbox/concat_2:0')
        )
        lbbox = tf.saved_model.utils.build_tensor_info(
            graph.get_tensor_by_name('pred_lbbox/concat_2:0')
        )

        labeling_signature = (
            tf.saved_model.signature_def_utils.build_signature_def(
                inputs={
                    "inputs": image_inputs,
                },
                outputs={
                    "sbbox": sbbox,
                    "mmbox": mbbox,
                    "lbbox": lbbox,
                },
                method_name=tf.saved_model.signature_constants
                .PREDICT_METHOD_NAME
            )
        )
        legacy_init_op = tf.group(
            tf.tables_initializer(),
            name='legacy_init_op'
        )
        builder.add_meta_graph_and_variables(
            sess,
            [tf.saved_model.tag_constants.SERVING],
            signature_def_map={'predict': labeling_signature},
            legacy_init_op=legacy_init_op,
            clear_devices=True
        )

        builder.save()
        DESTINATION = export_path.decode() + '/category.txt'
        copyfile(LABELS_FILE, DESTINATION)
        self.package_model_into_zip_file(export_path_base, export_path)

    def checkpoint2pb(self):
        graph = tf.Graph()
        with graph.as_default():
            with tf.name_scope('input'):
                input_data = tf.placeholder(dtype=tf.float32, name='input_data')
            model = YOLOV3(input_data, trainable=False)
            _pred_sbbox, _pred_mbbox, _pred_lbbox = \
                model.pred_sbbox, model.pred_mbbox, model.pred_lbbox
            print(_pred_sbbox, _pred_mbbox, _pred_lbbox)

            ckpt_path = tf.train.latest_checkpoint(FLAGS.lico_checkpoint_dir)

            with tf.Session(config=tf.ConfigProto(allow_soft_placement=True)) \
                    as sess:
                init = tf.global_variables_initializer()
                sess.run(init)
                # Restore variables from checkpoint file into graph
                saver = tf.train.Saver()
                saver.restore(sess, ckpt_path)
                self.save_pb(sess, FLAGS.lico_pb_path, graph)


if __name__ == '__main__':
    od_obj = YOLOv3ExportService()
    od_obj.checkpoint2pb()
